import java.awt.Dimension;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.ImageIcon;

import java.awt.Font;
import java.awt.Color;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class generates the about dialog. It shows some additional
 * information about the program and licenses. 
 * 
 * @author Daniel Brandstetter
 *
 */
public class AboutDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	JLabel image;
	JLabel lblGedichteeditor;
	JLabel lblVersion;
	JLabel lblVersionNumber;
	JLabel lblLetztenderung;
	JLabel lblLetztenderung_date;
	
	
	/**
	 * Create the dialog.
	 * @param ge: access to the main frame  
	 */
	public AboutDialog(GedichteEditor ge) {
		
		setResizable(false);
		setName("aDialog");
		setTitle(ge.BUNDLE.getString("AboutDialog.title")+" GedichteEditor");
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setPreferredSize(new Dimension(420, 220));
		
		getContentPane().setLayout(null);
		
		image = new JLabel("");
		image.setIcon(new ImageIcon(this.getClass().getResource("/resource/poem_icon.png")));
		image.setBounds(25, 25, 110, 100);
		getContentPane().add(image);
		
		lblGedichteeditor = new JLabel("GedichteEditor");
		lblGedichteeditor.setForeground(Color.DARK_GRAY);
		lblGedichteeditor.setFont(new Font("Tahoma", Font.PLAIN, 26));
		lblGedichteeditor.setBounds(170, 25, 206, 33);
		getContentPane().add(lblGedichteeditor);
		
		lblVersion = new JLabel(ge.BUNDLE.getString("AboutDialog.version"));
		lblVersion.setBounds(170, 74, 46, 14);
		getContentPane().add(lblVersion);
		
		lblVersionNumber = new JLabel("1.0.10");
		lblVersionNumber.setBounds(215, 74, 74, 14);
		getContentPane().add(lblVersionNumber);
		
		lblLetztenderung = new JLabel(ge.BUNDLE.getString("AboutDialog.lastchange"));
		lblLetztenderung.setBounds(170, 95, 95, 14);
		getContentPane().add(lblLetztenderung);
		
		lblLetztenderung_date = new JLabel("1.0.10");
		lblLetztenderung_date.setBounds(261, 95, 115, 14);
		getContentPane().add(lblLetztenderung_date);
		
		JLabel lblNewLabel = new JLabel("Copyright 2013 Daniel Brandstetter");
		lblNewLabel.setBounds(170, 120, 170, 14);
		getContentPane().add(lblNewLabel);
		
		JLabel lblNewLabel_1 = new JLabel("Lizenz GNU GPL");
		lblNewLabel_1.setBounds(170, 145, 119, 14);
		getContentPane().add(lblNewLabel_1);
		
		//Set Text
		setProjectData();
		
		pack();
	}
	
	/**
	 * Sets the version and last build date
	 */
	public void setProjectData() {
		
		Properties prop = new Properties();
		 
    	try {
    		
    		prop.load(new FileInputStream("appinfo.properties"));
    		this.lblVersionNumber.setText(prop.getProperty("program.VERSION")+"."+prop.getProperty("program.BUILDNUM"));
    		this.lblLetztenderung_date.setText(prop.getProperty("program.BUILDDATE"));
 
    	} catch (IOException ex) {
    		System.out.println("IOException" + ex);
        }
	}
}
