import java.awt.Color;
import java.awt.Dimension;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class processes the metrics. The dialog provides the user
 * the possibility to enter the emphasis of the metrics.
 * 
 * @author Daniel Brandstetter
 *
 */
public class AccentuationDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	GedichteEditor ge;
	Metric me;
	
	int line;
	int index = 1;
	int enter = 0;
	int count;
	String text_line = "";
	List<String> metric = new ArrayList<String>();
	
	JButton btnBetont;
	JButton btnUnbetont;
	
	/**
	 * Contstructor 
	 * @param ge: access to the main frame  
	 * @param line: line which is analyzed
	 */
	public AccentuationDialog(GedichteEditor ge, int line) {
		
		this.ge = ge;
		this.line = line;
		me = new Metric(ge, ge.wordlist);
		
		setLayout();
		initialize();
	}

	/**
	 * Sets the layout of the dialog
	 */
	private void setLayout() {
		
		setPreferredSize(new Dimension(240, 110));
		setModal(true);
		setName("aDialog");
		setTitle("Betonung");
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		
		getContentPane().setLayout(null);
		
		JPanel panel = new JPanel();
		panel.setBounds(10, 11, 204, 57);
		getContentPane().add(panel);
		panel.setLayout(null);
		
		btnBetont = new JButton(ge.BUNDLE.getString("AccentuationDialog.stressed"));
		btnBetont.addActionListener(this);
		btnBetont.setBounds(10, 11, 87, 35);
		panel.add(btnBetont);
		
		btnUnbetont = new JButton(ge.BUNDLE.getString("AccentuationDialog.unstressed"));
		btnUnbetont.addActionListener(this);
		btnUnbetont.setBounds(107, 11, 87, 35);
		panel.add(btnUnbetont);
		
		pack();
		
	}

	/**
	 * Marks the first syllable of the text line
	 */
	private void initialize() {
		this.count = me.getHyphCount(this.line);
		setStyledRhymeIndex(line, index);
		this.index ++;
	}

	/**
	 * Button action listener. Processes the input stress
	 */
	@SuppressWarnings("unchecked")
	public void actionPerformed(ActionEvent e) {
		
		if (e.getSource() == this.btnBetont) {
			if (this.index <= this.count) {
				setStyledRhymeIndex(this.line, this.index);
				this.index ++;
				metric.add("-");
			}
			
			this.enter ++;
			if (this.enter == this.count) {
				metric.add("-");
				String txtMe = me.getMetric(this.metric);
				
				if (txtMe != "") {
					ge.listModelMetric.set(this.line, this.line + 1 +": "+count+" "+ge.BUNDLE.getString("Default.syllables")+": "+txtMe);
				}
				
				ge.textList.put(this.line, this.text_line);
				
				String m = "";
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
				}
				ge.accentuationList.put(this.line, m);
				
				this.dispose();
				restStyleTextBackground();
			}
		}
		
		else if  (e.getSource() == this.btnUnbetont) {
			if (this.index <= this.count) {
				setStyledRhymeIndex(this.line, this.index);
				this.index ++;
				metric.add("e");
			}
			
			this.enter ++;
			if (this.enter == this.count) {
				metric.add("e");
				String txtMe = me.getMetric(this.metric);
				
				if (txtMe != "") {
					ge.listModelMetric.set(this.line, this.line + 1 +": "+count+" "+ge.BUNDLE.getString("Default.syllables")+": "+txtMe);
				}
				
				ge.textList.put(this.line, this.text_line);
				
				String m = "";
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
				}
				ge.accentuationList.put(this.line, m);
				
				this.dispose();
				restStyleTextBackground();
			}
		}
	}
	
	/**
	 * Those marked syllable, for which the user enters the emphasis.
	 * @param line: text line being analyzed
	 * @param index: index, which being examined syllable
	 */
	public void setStyledRhymeIndex(int line, int index) {
		
		restStyleTextBackground();
		
		Color green = new Color(212, 248, 198);        
        StyledDocument doc = ge.editor.getStyledDocument();
        int start = 0;
        int end = 0;
        int count = 0;
        int start_index = 0;
        int end_index = 0;
        String text = "";
        String[] words = null;
        Style style = ge.editor.addStyle("MyHilite", null);
        
        int length = ge.editor.getDocument().getLength();
        try {
			text = ge.editor.getDocument().getText(0, length);
		} catch (BadLocationException e) {
			System.out.println(e);
		}
        

        //Only one line
        if (text.indexOf("\n", start_index) == -1) {
        	start = 0;
        	end = text.length();
        }
        
        while (text.indexOf("\n", start_index) != -1) {
        
        	if (count == line) {
        		
        		start = start_index;
        		
        		end_index = text.indexOf("\n", start_index+1);
        		if (end_index != -1) end = end_index;
        		else end = text.length();
        		break;
        	}
        	
        	start_index = text.indexOf("\n", start_index + 1);
        	count ++;
        }
        
        if (line > 0) start = start+1;
        
        text_line = text.substring(start, end);
        words = text_line.split(" ");
        String hyph = "";
        String[] hyphList = null;
        int wordFullLength = 0;
        int start_hyph = 0;
        int c_new = 1;
        
        
        for (int i = 0; i < words.length; i++) {
            
            wordFullLength = words[i].length();

            String word = words[i].replaceAll("[^a-zA-Z]", "");
            if (ge.wordlist.get(word) != null) {
            	hyph = ge.wordlist.get(word);
            	hyphList = hyph.split("-");
            } else {
            	hyphList = word.split("  ");
            }
            
            start_hyph = start;
            
        	for (int x = 0; x < hyphList.length; x++) {

        		if (c_new == index) {
        			StyleConstants.setBackground(style, green);
        			doc.setCharacterAttributes(start_hyph, hyphList[x].length(), style, false);
        		}
        		
                start_hyph = start_hyph + hyphList[x].length();
                c_new ++;
        	}

        	start = start + wordFullLength + 1;       		
        }
	}

	/**
	 * Resets all styled text backgrounds
	 */
	public void restStyleTextBackground() {
		
		StyledDocument doc = ge.editor.getStyledDocument();
        Style style = ge.editor.addStyle("MyHilite", null);
        StyleConstants.setBackground(style, Color.white);
        doc.setCharacterAttributes(0, ge.editor.getText().length(), style, false);
	}
}
