import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class loads the dictionary and stores it an a hashmap.
 * 
 * @author Daniel Brandstetter
 *
 */
public class Dictionary {
	
	private String dictPath = "resource/wortliste";
	private Map<String,String> wordlist = new HashMap<String,String>();
	
	/**
	 * Constructor
	 */
	public Dictionary () {
		readDictionary();
	}
	
	/**
	 * Reads the dictionary from the file. It's always a key, pair value. 
	 * The key is the word, and the value is the hyphenation.
	 */
	private void readDictionary() {
		
		try {
			
			InputStream f = this.getClass().getResourceAsStream(dictPath);
			BufferedReader br = new BufferedReader(new InputStreamReader(f, "UTF-8"));
			
			String strLine;
			Matcher m = Pattern.compile("[^a-zA-Z]").matcher("");

			while ((strLine = br.readLine()) != null)   {
				
				String[] splitResult = strLine.split(";");
				wordlist.put(splitResult[0], m.reset(splitResult[splitResult.length-1]).replaceAll("-"));
				
			}

			br.close();
			
		} catch (FileNotFoundException e) {
			System.err.print(e);
		} catch (IOException e) {
			System.err.print(e);
		}
		
	}
	
	/**
	 * Getter for the wordlist
	 * @return Returns the wordlist
	 */
	public Map<String, String> getWordlist() {
		return this.wordlist;
	}

}