import java.awt.Dimension;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import java.awt.Insets;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;
import javax.swing.UIManager;
import javax.swing.JCheckBox;
import javax.swing.border.LineBorder;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.border.EmptyBorder;
import java.awt.Color;
import javax.swing.border.CompoundBorder;
import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import javax.swing.text.BadLocationException;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class generates the search dialog. The user can search for 
 * words in his text.
 * 
 * @author Daniel Brandstetter
 *
 */
public class FindDialog extends JDialog implements ActionListener, CaretListener {

	private static final long serialVersionUID = 1L;
	private GedichteEditor ge;
	
	JLabel lblFind;
	JTextField txtFind;
	JButton btnFind;
	JButton btnCancel;
	JPanel panel;
	JRadioButton rdbtnTop;
	JRadioButton rdbtnDown;
	ButtonGroup groupSearch;
	JCheckBox chckbxLowUpperCase;

	/**
	 * Constructor
	 * @param ge: access to the main frame 
	 */
	public FindDialog(GedichteEditor ge) {
		setName("fDialog");
		
		this.ge = ge;
		setLayout();
	}

	/**
	 * Sets the layout of the dialog
	 */
	private void setLayout() {
		
		setTitle(ge.BUNDLE.getString("FindDialog.title"));
		setResizable(false);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setPreferredSize(new Dimension(360, 130));
		
		getContentPane().setLayout(null);
		
		lblFind = new JLabel(ge.BUNDLE.getString("FindDialog.label.text"));
		lblFind.setDisplayedMnemonic(KeyEvent.VK_S);
		lblFind.setBounds(6, 7, 89, 26);
		getContentPane().add(lblFind);
		
		txtFind = new JTextField();
		lblFind.setLabelFor(txtFind);
		txtFind.setBorder(new CompoundBorder(new LineBorder(new Color(51, 153, 255)), new EmptyBorder(1, 1, 1, 1)));
		txtFind.setBounds(91, 10, 177, 20);
		getContentPane().add(txtFind);
		txtFind.setColumns(10);
		txtFind.addCaretListener(this);
		
		btnFind = new JButton(ge.BUNDLE.getString("FindDialog.button.search.text"));
		btnFind.addActionListener(this);
		btnFind.setMnemonic(KeyEvent.VK_W);
		btnFind.setMargin(new Insets(0, 0, 0, 0));
		btnFind.setEnabled(false);
		btnFind.setBounds(273, 9, 77, 23);
		getContentPane().add(btnFind);
		
		btnCancel = new JButton(ge.BUNDLE.getString("FindDialog.button.cancel.text"));
		btnCancel.addActionListener(this);
		btnCancel.setMargin(new Insets(0, 0, 0, 0));
		btnCancel.setBounds(273, 38, 77, 23);
		getContentPane().add(btnCancel);
		
		panel = new JPanel();
		panel.setBorder(new TitledBorder(UIManager.getBorder("TitledBorder.border"), ge.BUNDLE.getString("FindDialog.caption.option"), TitledBorder.LEFT, TitledBorder.TOP, null, null));
		panel.setBounds(160, 41, 108, 54);
		getContentPane().add(panel);
		panel.setLayout(null);
		
		rdbtnTop = new JRadioButton(ge.BUNDLE.getString("FindDialog.search.direction.up"));
		rdbtnTop.setMnemonic('o');
		rdbtnTop.setMargin(new Insets(0, 0, 0, 0));
		rdbtnTop.setBounds(6, 15, 96, 16);
		panel.add(rdbtnTop);
		
		rdbtnDown = new JRadioButton(ge.BUNDLE.getString("FindDialog.search.direction.down"));
		rdbtnDown.setMnemonic(KeyEvent.VK_U);
		rdbtnDown.setSelected(true);
		rdbtnDown.setMargin(new Insets(0, 0, 0, 0));
		rdbtnDown.setBounds(6, 34, 96, 14);
		panel.add(rdbtnDown);
		
		groupSearch = new ButtonGroup();
		groupSearch.add(rdbtnTop);
		groupSearch.add(rdbtnDown);
		
		chckbxLowUpperCase = new JCheckBox(ge.BUNDLE.getString("FindDialog.upperlowercase"));
		chckbxLowUpperCase.setMnemonic(KeyEvent.VK_C);
		chckbxLowUpperCase.setMargin(new Insets(0, 0, 0, 0));
		chckbxLowUpperCase.setBounds(6, 67, 144, 23);
		getContentPane().add(chckbxLowUpperCase);
		
		this.getRootPane().setDefaultButton(btnFind);
		
		pack();
		
	}

	/**
	 * Actions listener: Provides the search request.
	 */
	public void actionPerformed(ActionEvent e) {
		
		if(e.getSource() == this.btnCancel) {
			this.dispose();
		}
		
		if(e.getSource() == this.btnFind) {
			
			int pos = ge.editor.getCaretPosition();
			int se_end = ge.editor.getSelectionEnd();
			int se_start = ge.editor.getSelectionStart();
			String word = "";
			int index = -1;
			
			int length = ge.editor.getDocument().getLength();
			String text = "";
			try {
				text = ge.editor.getDocument().getText(0, length);
			} catch (BadLocationException e1) {
				System.out.println(e1);
			}

			if (se_end > pos) pos = se_end;
			if (this.chckbxLowUpperCase.isSelected()) {
				word = txtFind.getText();
			} else {
				word = txtFind.getText().toLowerCase();
				text = text.toLowerCase();
			}

			if (this.rdbtnDown.isSelected()) {
				index = text.indexOf(word, pos);			
			} else if (this.rdbtnTop.isSelected()) {
				index = text.lastIndexOf(word, se_start-1);
			}
			
			if (index == -1) {
				JOptionPane.showMessageDialog(null, "\""+word+"\" "+ge.BUNDLE.getString("FindDialog.info.text"), "GedichteEditor", JOptionPane.INFORMATION_MESSAGE);
			} else {
				ge.editor.select(index, index + word.length());
			}
		}
	}

	/**
	 * Enables, disables the search button. Depending on whether a character is 
	 * in the search field
	 */
	public void caretUpdate(CaretEvent e) {
		
		if(e.getSource() == this.txtFind) {
			
			//Get text
			String text = txtFind.getText();
			
			if ( text == null || text.isEmpty()) {
				btnFind.setEnabled(false);
			} else {
				btnFind.setEnabled(true);
			}
		}
	}
}
