import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JTextPane;
import javax.swing.UIManager;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JToolBar;
import java.awt.BorderLayout;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.KeyStroke;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import javax.swing.JCheckBox;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import java.awt.Dimension;

import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.SystemColor;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;

import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import javax.swing.event.MenuListener;
import javax.swing.event.MenuEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;
import javax.swing.undo.UndoManager;


import javax.swing.border.LineBorder;
import javax.swing.border.CompoundBorder;
import java.awt.Color;
import javax.swing.border.EmptyBorder;
import java.awt.FlowLayout;
import javax.swing.JButton;
import javax.swing.SwingConstants;
import javax.swing.JSeparator;
import javax.swing.border.MatteBorder;
import javax.swing.JLabel;
import javax.swing.JList;
import java.awt.Component;
import java.awt.Font;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.ScrollPaneConstants;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import javax.swing.JSplitPane;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class is the main class of GedichteEditor. It processes any 
 * initialisation and user inputs, and sets the language of the program.
 * It contains the main user interface.
 * 
 * @author Daniel Brandstetter
 *
 */
public class GedichteEditor implements ActionListener, UndoableEditListener, CaretListener,
	MenuListener, MouseListener {
	
	//Classes
	public Dictionary dictionary;
	public ActionPerform action;
	public UndoManager undoManager;
	public UndoAction undoAction;
	public RedoAction redoAction;
	public Metric me;
	
	Clipboard sysClip = Toolkit.getDefaultToolkit().getSystemClipboard();

	//Frame
	JFrame frmGedichteeditor;
	
	//Menu
	JMenu file;
	JMenu edit;
	JMenu rhyme;
	JMenu help;
	
	//Menu -> File
	JMenuItem newfile;
	JMenuItem open;
	JMenuItem save;
	JMenuItem saveas;
	JMenuItem print;
	JMenuItem exit;
	
	//Menu -> Edit
	JMenuItem copy;
	JMenuItem cut;
	JMenuItem paste;
	JMenuItem delete;
	JMenuItem find;
	JMenuItem selectall;
	
	//Menu -> Metric
	JMenu metric;
	
	//Menu -> Rhyme
	JMenu subMenuOne_;
	JMenu subMenuTwo_;
	JMenu subMenuThree_;
	JMenuItem template;

	//Menu -> Help
	JMenuItem help_;
	JMenu language;
	JMenuItem about;
	JCheckBoxMenuItem en;
	JCheckBoxMenuItem de;
	
	//Toolbar
	JToolBar toolBar;
	JCheckBox chckbxSilbentrennung;
	JButton btnNew;
	JSeparator separator;
	JButton btnOpen;
	JButton btnSave;
	
	//Content Area
	JPanel panel_middle;
	JScrollPane scrollPane_panel;
	JPanel panel_txt;
	JTextPane editor;
	
	//Popup Menu
	JPopupMenu popupMenu;
	JMenuItem hyphenation;
	JMenu subMenuOne;
	JMenu subMenuTwo;
	JMenu subMenuThree;
	
	//Metric Bar
	JPanel panelMetric;
	JPanel panelMetricTitle;
	JPanel panelMetricList;
	JPanel panelMetricTitleLeft;
	JPanel panelMetricTitleRight;
	JLabel lblMetricTitle;
	JButton btnMetricClose;
	JCheckBoxMenuItem showMetric;
	JScrollPane scrollPaneMetric;
	@SuppressWarnings("rawtypes")
	DefaultListModel listModelMetric;
	@SuppressWarnings("rawtypes")
	JList listMetric;
	JButton btnUpdateMetric;
	
	//Variablen
	public Map<String, String> wordlist;
	private String oldWordMenu = "";
	private String oldWordMouse = "";
	private String textOld = "";
	private JButton btnMetricAnalyse;
	private JButton btnTemplate;
	public Map<Integer, String> textList = new HashMap<Integer, String>();
	public Map<Integer, String> accentuationList = new HashMap<Integer, String>();
	public String currentLanguage = "";
	public ResourceBundle BUNDLE;
	private JSplitPane splitPane;
	private int dividerLocation = 0;
	private JPanel panelStatus;
	private JLabel lblLineCount;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		
		System.setProperty("sun.java2d.noddraw", Boolean.TRUE.toString());
		
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch(Exception e) {
			System.out.println("Unknown Look and Feel." + e); 
		}

		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					new GedichteEditor();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public GedichteEditor() {
		setLanguage();
		initialize();
		setLayout();
	}

	/**
	 * Sets the default language, looks for the set language in 
	 * property file.
	 */
	private void setLanguage() {
		
		Properties prop = new Properties();
		Locale currentLocale = null;
		
		File file = new File("config.properties");
		if(file.exists()) {
		
			try {
				prop.load(new FileInputStream("config.properties"));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		
			if (prop.getProperty("language").equals("en")) {
				currentLocale = new Locale("en","GB","");
				this.currentLanguage = "en";
				Locale.setDefault(Locale.ENGLISH);
			} else if (prop.getProperty("language").equals("de")) {
				currentLocale = new Locale("de","DE","");
				this.currentLanguage = "de";
				Locale.setDefault(Locale.GERMAN);
			} else {
				currentLocale = new Locale("de","DE","");
				this.currentLanguage = "de";
				Locale.setDefault(Locale.GERMAN);
			}
		} else {
			currentLocale = new Locale("de","DE","");
			this.currentLanguage = "de";
			Locale.setDefault(Locale.GERMAN);
		}
		
		BUNDLE = ResourceBundle.getBundle("i18n.language", currentLocale);
	}
	
	/*
	 * If user changes the language, the new language is stored
	 * in property file.
	 */
	public void setNewLanguage() {
		
		Properties prop = new Properties();
		prop.setProperty("language", this.currentLanguage);
		
		File file = new File("config.properties");
		if(!file.exists()) {
		    try {
				file.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} 
		
		try {
			prop.store(new FileOutputStream("config.properties"), null);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	

	/**
	 * Initialize the classes, and set the wordlist
	 */
	private void initialize() {
		
		dictionary = new Dictionary();
		action = new ActionPerform(this, dictionary);
		undoManager = new UndoManager();
		undoAction = new UndoAction(this);
		redoAction = new RedoAction(this);
		me = new Metric(this, dictionary.getWordlist());
		
		//hyph = new Hyphenation(editor, this, dictionary.getWordlist());
		this.wordlist = dictionary.getWordlist();
	}
	
	/**
	 * Creates all user interface elements
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void setLayout() {
		
		frmGedichteeditor = new JFrame();		
		frmGedichteeditor.setPreferredSize(new Dimension(1070, 700));
		frmGedichteeditor.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				action.exitApplication();
			}
		});
		frmGedichteeditor.setIconImage(Toolkit.getDefaultToolkit().getImage(this.getClass().getResource("/resource/poem_icon.png")));
		frmGedichteeditor.setTitle("GedichteEditor");
		
		JMenuBar menuBar = new JMenuBar();
		menuBar.setDoubleBuffered(true);
		frmGedichteeditor.setJMenuBar(menuBar);
		
		//File
		file = new JMenu(BUNDLE.getString("GedichteEditor.menu.file"));
		menuBar.add(file);
		
		//File -> New
		newfile = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.file.new"));
		newfile.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		newfile.addActionListener(this);
		file.add(newfile);
		
		//File -> Open 
		open = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.file.open"));
		open.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		open.addActionListener(this);
		file.add(open);
		
		//File -> Save
		save = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.file.save"));
		save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		save.addActionListener(this);
		file.add(save);
		
		//File -> SaveAs
		saveas = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.file.saveas"));
		saveas.addActionListener(this);
		file.add(saveas);
		
		//File -> Print
		print = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.file.print"));
		print.addActionListener(this);
		print.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK));
		file.add(print);
		
		//File -> Exit
		exit = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.file.exit"));
		exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, InputEvent.ALT_MASK));
		exit.addActionListener(this);
		file.add(exit);
		
		//File -> Seperator
		file.insertSeparator(4);
		file.insertSeparator(6);

		//Edit
		edit = new JMenu(BUNDLE.getString("GedichteEditor.menu.edit"));
		edit.addMenuListener(this);
		menuBar.add(edit);
		
		//Edit -> Undo
		edit.add(undoAction);
		
		//Edit -> Redo
		edit.add(redoAction);
		
		//Edit -> Cut
		cut = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.edit.cut"));
		cut.setEnabled(false);
		cut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
		cut.addActionListener(this);
		edit.add(cut);

		//Edit -> Copy
		copy = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.edit.copy"));
		copy.setEnabled(false);
		copy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK));
		copy.addActionListener(this);
		edit.add(copy);
		
		//Edit -> Paste
		paste = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.edit.paste"));
		paste.setEnabled(false);
		paste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));
		paste.addActionListener(this);
		edit.add(paste);
		
		//Edit -> Delete
		delete = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.edit.delete"));
		delete.setEnabled(false);
		delete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		delete.addActionListener(this);
		edit.add(delete);
		
		//Edit -> SelectAll
		selectall = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.edit.selectall"));
		selectall.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK));
		selectall.addActionListener(this);
		edit.add(selectall);
		
		//Edit -> Find
		find = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.edit.search"));
		find.setEnabled(false);
		find.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK));
		find.addActionListener(this);
		edit.add(find);
		
		//Edit -> Seperator
		edit.insertSeparator(2);
		edit.insertSeparator(7);
		
		//Metric
		metric = new JMenu(BUNDLE.getString("GedichteEditor.menu.metric"));
		metric.addMenuListener(this);
		menuBar.add(metric);
		
		showMetric = new JCheckBoxMenuItem(BUNDLE.getString("GedichteEditor.menu.metric.analyse"));
		showMetric.addActionListener(this);
		showMetric.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK));
		metric.add(showMetric);
		
		
		//Rhyme
		rhyme = new JMenu(BUNDLE.getString("GedichteEditor.menu.rhyme"));
		rhyme.addMenuListener(this);
		menuBar.add(rhyme);
		subMenuOne_ = new JMenu(BUNDLE.getString("GedichteEditor.menu.rhyme.syllableone"));
		MenuScroller.setScrollerFor(subMenuOne_, 10, 100, 0, 0);
		subMenuTwo_ = new JMenu(BUNDLE.getString("GedichteEditor.menu.rhyme.syllabletwo"));
		MenuScroller.setScrollerFor(subMenuTwo_, 10, 100, 0, 0);
		subMenuThree_ = new JMenu(BUNDLE.getString("GedichteEditor.menu.rhyme.syllablemore"));
		MenuScroller.setScrollerFor(subMenuThree_, 10, 100, 0, 0);
		rhyme.add(subMenuOne_);
		rhyme.add(subMenuTwo_);
		rhyme.add(subMenuThree_);
		subMenuOne_.setEnabled(false);
		subMenuTwo_.setEnabled(false);
		subMenuThree_.setEnabled(false);
		
		//Rhyme -> Seperator
		rhyme.insertSeparator(3);
		
		//Rhyme -> Template
		template = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.rhyme.template"));
		template.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_MASK));
		template.addActionListener(this);
		rhyme.add(template);
		
		//Help
		help = new JMenu(BUNDLE.getString("GedichteEditor.menu.help"));
		menuBar.add(help);
		
		//Help -> Help
		help_ = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.help"));
		help_.addActionListener(this);
		help_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0)); 
		help.add(help_);
		
		//Help -> Language
		language = new JMenu(BUNDLE.getString("GedichteEditor.menu.help.language"));
		en = new JCheckBoxMenuItem(BUNDLE.getString("GedichteEditor.menu.help.languageen"));
		en.addActionListener(this);
		de = new JCheckBoxMenuItem(BUNDLE.getString("GedichteEditor.menu.help.languagede"));
		de.addActionListener(this);
		language.add(en);
		language.add(de);
		help.add(language);
		if (this.currentLanguage.equals("de")) de.setSelected(true);
		else en.setSelected(true);
		
		//Help -> About
		about = new JMenuItem(BUNDLE.getString("GedichteEditor.menu.help.about")+" GedichteEditor");
		about.addActionListener(this);
		help.add(about);
	
		// Sets the mnemonic to the menu.
		file.setMnemonic('d');
		edit.setMnemonic('b');
		metric.setMnemonic('m');
		showMetric.setMnemonic('t');
		rhyme.setMnemonic('r');
		help.setMnemonic('h');
		newfile.setMnemonic('n');
		open.setMnemonic('');
		save.setMnemonic('s');
		saveas.setMnemonic('u');
		print.setMnemonic('d');
		exit.setMnemonic('b');
		cut.setMnemonic('a');
		copy.setMnemonic('k');
		paste.setMnemonic('e');
		selectall.setMnemonic('m');
		find.setMnemonic('s');
		delete.setMnemonic('l');
		about.setMnemonic('e');
		template.setMnemonic('v');
		subMenuOne_.setMnemonic('e');
		subMenuTwo_.setMnemonic('z');
		subMenuThree_.setMnemonic('m');
		help_.setMnemonic('h');
		language.setMnemonic('s');
		de.setMnemonic('d');
		en.setMnemonic('e');
		
		
		//Toolbar
		toolBar = new JToolBar();
		toolBar.setBorder(new MatteBorder(0, 0, 2, 0, (Color) Color.GRAY));
		toolBar.setRollover(true);
		toolBar.setBackground(SystemColor.control);
		
		btnNew = new JButton();
		btnNew.addActionListener(this);
		btnNew.setIcon(new ImageIcon(this.getClass().getResource("/resource/new.png")));
		toolBar.add(btnNew);
		
		btnOpen = new JButton();
		btnOpen.addActionListener(this);
		btnOpen.setIcon(new ImageIcon(this.getClass().getResource("/resource/open.png")));
		toolBar.add(btnOpen);
		
		btnSave = new JButton();
		btnSave.addActionListener(this);
		btnSave.setIcon(new ImageIcon(this.getClass().getResource("/resource/save.png")));
		toolBar.add(btnSave);
		
		separator = new JSeparator();
		separator.setMaximumSize(new Dimension(1, 30));
		separator.setPreferredSize(new Dimension(1, 10));
		separator.setOrientation(SwingConstants.VERTICAL);
		toolBar.add(separator);
		
		chckbxSilbentrennung = new JCheckBox("Silbentrennung");
		chckbxSilbentrennung.setBorder(new EmptyBorder(0, 10, 0, 0));
		chckbxSilbentrennung.setMargin(new Insets(2, 10, 2, 2));
		toolBar.add(chckbxSilbentrennung);
		chckbxSilbentrennung.addActionListener(this);
		chckbxSilbentrennung.setBackground(SystemColor.control);
		chckbxSilbentrennung.setVisible(false);
		
		panelStatus = new JPanel();
		panelStatus.setPreferredSize(new Dimension(10, 20));
		frmGedichteeditor.getContentPane().add(panelStatus, BorderLayout.SOUTH);
		panelStatus.setLayout(new BorderLayout(0, 0));
		
		lblLineCount = new JLabel("");
		lblLineCount.setBorder(new EmptyBorder(0, 20, 0, 0));
		panelStatus.add(lblLineCount, BorderLayout.WEST);
		
		
		frmGedichteeditor.getContentPane().add(toolBar, BorderLayout.NORTH);
		
		btnMetricAnalyse = new JButton(BUNDLE.getString("GedichteEditor.menu.metric.analyse"));
		btnMetricAnalyse.addActionListener(this);
		btnMetricAnalyse.setIcon(new ImageIcon(this.getClass().getResource("/resource/text.png")));
		toolBar.add(btnMetricAnalyse);
		
		btnTemplate = new JButton(BUNDLE.getString("GedichteEditor.toolbar.rhymetemplate"));
		btnTemplate.addActionListener(this);
		btnTemplate.setIcon(new ImageIcon(this.getClass().getResource("/resource/template.png")));
		toolBar.add(btnTemplate);
		
		splitPane = new JSplitPane();
		splitPane.setDividerSize(0);
		frmGedichteeditor.getContentPane().add(splitPane, BorderLayout.CENTER);
		
		panelMetric = new JPanel();
		splitPane.setRightComponent(panelMetric);
		panelMetric.setVisible(false);		
		
		//Content Area
		panel_middle = new JPanel();
		splitPane.setLeftComponent(panel_middle);
		panel_middle.setLayout(new BorderLayout(0, 0));
		
		scrollPane_panel = new JScrollPane();
		scrollPane_panel.getVerticalScrollBar().setUnitIncrement(14);
		panel_middle.add(scrollPane_panel);
		
		panel_txt = new JPanel();
		panel_txt.setBackground(SystemColor.scrollbar);
		panel_txt.setBorder(new CompoundBorder(null, new EmptyBorder(10, 5, 5, 5)));
		scrollPane_panel.setViewportView(panel_txt);
		panel_txt.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
		
		
		//EditorPane
		editor = new JTextPane();
		editor.setFont(new Font("Calibri", Font.PLAIN, 16));
		editor.addFocusListener(new FocusAdapter() {
			@Override
			public void focusGained(FocusEvent arg0) {
				me.restStyleTextBackground();
			}
		});
		
		editor.getDocument().addUndoableEditListener(this);
		editor.addMouseListener(this);
		editor.setDoubleBuffered(true);
		editor.addCaretListener(this);
		editor.setMargin(new Insets(10, 3, 3, 3));
		editor.setBorder(new CompoundBorder(new LineBorder(new Color(180, 180, 180), 15), new EmptyBorder(10, 20, 20, 20)));
		editor.setPreferredSize(new Dimension(800, 8000));
		editor.setCaret(new DefaultCaret() {

			private static final long serialVersionUID = 1L;
			public void paint(Graphics g) {

		        JTextComponent comp = getComponent();
		        if (comp == null)
		            return;

		        Rectangle r = null;
		        try {
		            r = comp.modelToView(getDot());
		            if (r == null)
		                return;
		        } catch (BadLocationException e) {
		            return;
		        }
		        r.height = 18; //this value changes the caret size
		        if (isVisible())
		            g.fillRect(r.x, r.y, 1, r.height);
		    }
		});
		panel_txt.add(editor);
		editor.requestFocus();
		panelMetric.setBorder(new LineBorder(Color.GRAY));
		panelMetric.setPreferredSize(new Dimension(210, 10));
		//frmGedichteeditor.getContentPane().add(panelMetric, BorderLayout.EAST);
		panelMetric.setLayout(new BorderLayout(0, 0));
		
		panelMetricTitle = new JPanel();
		panelMetricTitle.setBorder(new LineBorder(Color.LIGHT_GRAY, 5));
		panelMetricTitle.setBackground(Color.GRAY);
		panelMetricTitle.setBounds(new Rectangle(0, 0, 200, 20));
		panelMetric.add(panelMetricTitle, BorderLayout.NORTH);
		panelMetricTitle.setLayout(new BorderLayout(0, 0));
		
		panelMetricTitleLeft = new JPanel();
		panelMetricTitleLeft.setBorder(new EmptyBorder(0, 5, 0, 5));
		panelMetricTitleLeft.setBackground(Color.GRAY);
		panelMetricTitle.add(panelMetricTitleLeft, BorderLayout.WEST);
		panelMetricTitleLeft.setLayout(new BorderLayout(0, 0));
		
		lblMetricTitle = new JLabel(BUNDLE.getString("GedichteEditor.metric.title"));
		lblMetricTitle.setFont(new Font("Tahoma", Font.PLAIN, 12));
		lblMetricTitle.setForeground(Color.WHITE);
		lblMetricTitle.setAlignmentX(Component.CENTER_ALIGNMENT);
		panelMetricTitleLeft.add(lblMetricTitle);
		
		panelMetricTitleRight = new JPanel();
		panelMetricTitleRight.setBackground(Color.GRAY);
		panelMetricTitle.add(panelMetricTitleRight, BorderLayout.EAST);
		
		btnMetricClose = new JButton("x");
		btnMetricClose.addActionListener(this);
		
		btnUpdateMetric = new JButton(BUNDLE.getString("GedichteEditor.metric.update"));
		btnUpdateMetric.addActionListener(this);
		btnUpdateMetric.setPreferredSize(new Dimension(93, 18));
		panelMetricTitleRight.add(btnUpdateMetric);
		
		btnMetricClose.setFont(new Font("Tahoma", Font.PLAIN, 9));
		btnMetricClose.setPreferredSize(new Dimension(18, 18));
		btnMetricClose.setMargin(new Insets(2, 5, 2, 5));
		panelMetricTitleRight.add(btnMetricClose);
		
		panelMetricList = new JPanel();
		panelMetric.add(panelMetricList, BorderLayout.CENTER);
		panelMetricList.setLayout(new BorderLayout(0, 0));
		
		scrollPaneMetric = new JScrollPane();
		scrollPaneMetric.setBorder(new MatteBorder(0, 5, 0, 0, (Color) Color.LIGHT_GRAY));
		scrollPaneMetric.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPaneMetric.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		panelMetricList.add(scrollPaneMetric, BorderLayout.CENTER);
		
		listModelMetric = new DefaultListModel();
		listMetric = new JList(listModelMetric);
		listMetric.setFixedCellHeight(20);
		listMetric.setFont(listMetric.getFont().deriveFont(11.0f));
		listMetric.addMouseListener(this);
		scrollPaneMetric.setViewportView(listMetric);
		
		//Popup Menu
		popupMenu = new JPopupMenu();
		hyphenation = new JMenuItem();
		hyphenation.setEnabled(false);
		popupMenu.add(hyphenation);
		popupMenu.addSeparator();
		//Submenu
		subMenuOne = new JMenu(BUNDLE.getString("GedichteEditor.menu.rhyme.syllableone"));
		MenuScroller.setScrollerFor(subMenuOne, 10, 100, 0, 0);
		subMenuTwo = new JMenu(BUNDLE.getString("GedichteEditor.menu.rhyme.syllabletwo"));
		MenuScroller.setScrollerFor(subMenuTwo, 10, 100, 0, 0);
		subMenuThree = new JMenu(BUNDLE.getString("GedichteEditor.menu.rhyme.syllablemore"));
		MenuScroller.setScrollerFor(subMenuThree, 10, 100, 0, 0);
		
		subMenuOne.setEnabled(false);
		subMenuTwo.setEnabled(false);
		subMenuThree.setEnabled(false);
		
		popupMenu.add(subMenuOne);
		popupMenu.add(subMenuTwo);
		popupMenu.add(subMenuThree);
		
		this.btnMetricAnalyse.setMnemonic('e');
		this.btnTemplate.setMnemonic('m');
		this.btnUpdateMetric.setMnemonic('a');
		
		//Pack frame
		frmGedichteeditor.pack();
		frmGedichteeditor.setLocationRelativeTo(null);
		frmGedichteeditor.setVisible(true);
	}
	
	/**
	 * Returns the currently selected word in the main textpane
	 * @return
	 */
	private String getCurrentWord() {
		
		int pos = editor.getCaretPosition();
		int start = pos;
		int end = pos;
		String word = "";
		
		try {
			if (start > 0) start -= 1;
			while ((editor.getText(start, 1).matches("[a-zA-Z]+")) && (start > 0)) start -= 1;
			while ((editor.getText(end, 1).matches("[a-zA-Z]+")) && (end < (editor.getText().length()))) end += 1;
			
			word = editor.getText(start, end-start).trim();
			word = word.replaceAll("[^a-zA-Z]", "");
			
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}

		return word;
	}
	
	/**
	 * Gets the current line number at the cursor position
	 * @param comp
	 * @param offset
	 * @return the current line number
	 * @throws BadLocationException
	 */
	int getLineOfOffset(JTextComponent comp, int offset) throws BadLocationException {
	    Document doc = comp.getDocument();
	    if (offset < 0) {
	        throw new BadLocationException("Can't translate offset to line", -1);
	    } else if (offset > doc.getLength()) {
	        throw new BadLocationException("Can't translate offset to line", doc.getLength() + 1);
	    } else {
	        Element map = doc.getDefaultRootElement();
	        return map.getElementIndex(offset);
	    }
	}
	
	/**
	 * Processes all actions by the user.
	 */
	public void actionPerformed(ActionEvent e) {
		
		//File -> NewFile
		if(e.getSource() == this.newfile || e.getSource() == this.btnNew) {
			action.newFile();
        }
		
		//File -> Open
		else if(e.getSource() == this.open || e.getSource() == this.btnOpen) {
			action.openFile();
        }
		
		//File -> Save
		else if(e.getSource() == this.save || e.getSource() == this.btnSave) {
			action.saveFile();
        }
		
		//File -> SaveAs
		else if(e.getSource() == this.saveas) {
			action.saveFileAs();
        }
		
		//File -> Print
		else if(e.getSource() == this.print) {
			action.print();
        }

		//File -> Exit
		else if(e.getSource() == this.exit) {
			action.exitApplication();
        }
		
		//Edit -> Cut
		else if(e.getSource() == this.cut) {
			action.cutFile();
        }
		
		//Edit -> Copy
		else if(e.getSource() == this.copy) {
			action.copyFile();
        }
		
		//Edit -> Paste
		else if(e.getSource() == this.paste) {
			action.pasteFile();
        }
		
		//Edit -> Delete
		else if(e.getSource() == this.delete) {
			action.deleteSelectedText();
        }
		
		//Edit -> SelectAll
		else if(e.getSource() == this.selectall) {
			action.selectAll();
        }
		
		//Edit -> Find
		else if(e.getSource() == this.find) {
			action.findText();
        }
		
		//Metric -> showMetric
		else if(e.getSource() == this.showMetric || e.getSource() == this.btnMetricAnalyse || e.getSource() == this.btnMetricClose) {
			
			if (panelMetric.isVisible()) {
				this.splitPane.setDividerSize(0);
				dividerLocation = splitPane.getDividerLocation();
				panelMetric.setVisible(false);
				showMetric.setSelected(false);
				me.restStyleTextBackground();
			} else {
				splitPane.setDividerSize(5);
				if (dividerLocation == 0) {
					dividerLocation = this.frmGedichteeditor.getSize().width-this.frmGedichteeditor.getSize().width/(5);
				}
				splitPane.setDividerLocation(dividerLocation);
				
				panelMetric.setVisible(true);
				showMetric.setSelected(true);
				me.analyseText(editor.getText());
			}
		}
		
		//Textanalyse update Button
		else if (e.getSource() == this.btnUpdateMetric) {
			me.updateText(editor.getText());
			me.restStyleTextBackground();
		}
		
		//Rhyme -> Template
		else if (e.getSource() == this.template || e.getSource() == this.btnTemplate) {
			action.rhymeTemplate();
		}
		
		//Help -> Help
		else if (e.getSource() == this.help_) {
			
			try {
				Runtime.getRuntime().exec("hh.exe GedichteEditor.chm");
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
		
		//Help -> Language
		else if(e.getSource() == this.de) {
			this.currentLanguage = "de";
			setNewLanguage();
			this.en.setSelected(false);
			this.de.setSelected(true);
			JOptionPane.showMessageDialog(null, BUNDLE.getString("GedichteEditor.language.hint"), "GedichteEditor", JOptionPane.INFORMATION_MESSAGE);
		}
		else if(e.getSource() == this.en) {
			this.currentLanguage = "en";
			setNewLanguage();
			this.en.setSelected(true);
			this.de.setSelected(false);
			JOptionPane.showMessageDialog(null, BUNDLE.getString("GedichteEditor.language.hint"), "GedichteEditor", JOptionPane.INFORMATION_MESSAGE);
		}
		
		//Help -> About
		else if(e.getSource() == this.about) {
			action.about();
        }
	}

	/**
	 * Processes undo/redo action. If any undo/redo action is 
	 * possible, this event fires.
	 */
	public void undoableEditHappened(UndoableEditEvent e) {

		undoManager.addEdit(e.getEdit());
		undoAction.update();
		redoAction.update();	
	}

	/**
	 * Processes all caret updates of the main textpane.
	 */
	public void caretUpdate(CaretEvent e) {
		
		this.cut.setEnabled(false);
		this.copy.setEnabled(false);
		this.delete.setEnabled(false);
		this.find.setEnabled(false);
		
		//Selected Text
		if (editor.getSelectedText() != null) {
			this.cut.setEnabled(true);
			this.copy.setEnabled(true);
			this.delete.setEnabled(true);
		}
		
		//Text not null
		if (editor.getText() != null) {
			this.find.setEnabled(true);
		}
		
		if (panelMetric.isVisible()) {
			
			String text = editor.getText();
			
			if (!textOld.equals(text)) this.listMetric.setEnabled(false);
			textOld = text;
		}
		
		//Update Statusbar line count
		int dot = e.getDot();
		int line = 0;
	    try {
			line = getLineOfOffset(editor, dot);
		} catch (BadLocationException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	    line += 1;
		lblLineCount.setText(BUNDLE.getString("Default.line")+line);
	}

	//NOT NEEDED
	public void menuCanceled(MenuEvent e) {
	}
	public void menuDeselected(MenuEvent e) {
	}

	/**
	 * If user selects any menu item, this event fires.
	 * Updates the menu if needed, for example to load new
	 * rhyme words.
	 */
	@SuppressWarnings("null")
	public void menuSelected(MenuEvent e) {
		
		if(e.getSource() == this.edit) {
			
			//Check clipboard
			Transferable transfer = sysClip.getContents(null);
			String data = null;
			if (transfer.isDataFlavorSupported(DataFlavor.stringFlavor)) {
				try {
					data = (String) transfer.getTransferData(DataFlavor.stringFlavor );
				} catch (UnsupportedFlavorException ex) {
					System.out.println("UnsupportedFlavorException" + ex); 
				} catch (IOException ex) {
					System.out.println("IOException" + ex); 
				}
				
				if ((data == null) && (data.isEmpty())) paste.setEnabled(false);
				else paste.setEnabled(true);
			}
		}
		
		if (e.getSource() == this.rhyme) {
			
			String word = getCurrentWord();
			
			if (word != null && !word.equals("") && !word.equals(" ") && !oldWordMenu.equals(word)) {
				
				subMenuOne_.removeAll();
				subMenuTwo_.removeAll();
				subMenuThree_.removeAll();
				
				subMenuOne_.setEnabled(false);
				subMenuOne_.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				subMenuTwo_.setEnabled(false);
				subMenuTwo_.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				subMenuThree_.setEnabled(false);
				subMenuThree_.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				
				Rhyme r = new Rhyme(this, dictionary.getWordlist(), word);
				r.start();
				
				oldWordMenu = word;
			}
		}
	}
	
	
	/**
	 * Escape all Dialogs when ESC pressed
	 * @param dialog
	 */
	public void addEscapeListener(final JDialog dialog) {
	    ActionListener escListener = new ActionListener() {

	        @Override
	        public void actionPerformed(ActionEvent e) {
	            dialog.setVisible(false);
	        }
	    };

	    dialog.getRootPane().registerKeyboardAction(escListener,
	            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
	            JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	
	/**
	 * MouseListener for the metric list
	 */
	public void mouseClicked(MouseEvent e) {
		
		if (e.getSource() == this.listMetric && this.listMetric.isEnabled()) {
			JList<?> list = (JList<?>)e.getSource();
			
	        if (e.getClickCount() == 2) {
	        	int index = list.locationToIndex(e.getPoint());
	        	if (index != -1) {
	        		action.accentution(index);
	        		setLineNumber(index);
	        	}
	        	
	        } else if (e.getClickCount() == 1) {
	        	int index = list.locationToIndex(e.getPoint());
	            
	            if (index != -1) {
	    			me.styleTextBackground(index);
	    			setLineNumber(index);
	    		}
	            this.listMetric.setEnabled(true);
	        }
		}
        
	}
	
	private void setLineNumber(int line) {
	    
		String text = "";
		int start_index = 0;
        int end_index = 0;
        int start = 0;
        @SuppressWarnings("unused")
		int end = 0;
        int count = 0;
		int length = editor.getDocument().getLength();
		
        try {
			text = editor.getDocument().getText(0, length);
		} catch (BadLocationException e) {
			System.out.println(e);
		}
        
        if (text.indexOf("\n", start_index) == -1) {
        	start = 0;
        	end = text.length();
        }
        
        while (text.indexOf("\n", start_index) != -1) {
        
        	if (count == line) {
        		
        		start = start_index;
        		
        		end_index = text.indexOf("\n", start_index+1);
        		if (end_index != -1) end = end_index;
        		else end = text.length();
        		break;
        	}
        	
        	start_index = text.indexOf("\n", start_index + 1);
        	count ++;
        }
        
        if (line > 0) start = start+1;        
	    editor.setCaretPosition(start);
	}
	
	public void mouseEntered(MouseEvent e) {	
	}
	public void mouseExited(MouseEvent e) {	
	}
	public void mousePressed(final MouseEvent e) {
		me.restStyleTextBackground();
	}
	
	/**
	 * Loads the rhyme words on right click event.
	 */
	public void mouseReleased(MouseEvent e) {	
		
		if (e.getButton() == MouseEvent.BUTTON3 && e.getSource() == this.editor) {
			
			editor.setCaretPosition(editor.viewToModel(e.getPoint()));
			
			String word = getCurrentWord();
			
			//Hyphenation			
			if (wordlist.get(word) != null) {
				String w = wordlist.get(word);
				hyphenation.setText(w);
			}
			else hyphenation.setText(word);
			if (word == null || word.equals("") || word.equals(" ")) {
				hyphenation.setText(BUNDLE.getString("GedichteEditor.selectword"));
			}
			
			
			//Rhyme
			if (word != null && !word.equals("") && !word.equals(" ") && !oldWordMouse.equals(word)) {
				
				subMenuOne.removeAll();
				subMenuTwo.removeAll();
				subMenuThree.removeAll();
				
				subMenuOne.setEnabled(false);
				subMenuOne.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				subMenuTwo.setEnabled(false);
				subMenuTwo.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				subMenuThree.setEnabled(false);
				subMenuThree.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				
				Rhyme r = new Rhyme(this, dictionary.getWordlist(), word);
				r.start();
				
				oldWordMouse = word;
				
			}
			
			//Popup Menu
			popupMenu.show(editor, e.getX()+10, e.getY()+10);
			subMenuOne.setMenuLocation(subMenuOne.getWidth()+2, 0);
			subMenuTwo.setMenuLocation(subMenuTwo.getWidth()+2, 0);
			subMenuThree.setMenuLocation(subMenuThree.getWidth()+2, 0);
		}
	}
}
