import java.awt.print.PrinterJob;
import java.awt.print.PrinterException;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.RepaintManager;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class generates a print dialog
 * 
 * @author Daniel Brandstetter
 *
 */
public class PrintClass implements Printable {
	
	private Component compPrint;
	public GedichteEditor ge;
   
   
	/**
	 * Constructor
	 * @param compPrint: Component which contains the text
	 * @param ge: access to the main frame 
	 */
	public PrintClass (Component compPrint, GedichteEditor ge) {
		this.compPrint = compPrint;
		this.ge = ge;
	}
	
	/**
	 * Calls the printer methode, prints the text
	 */
	public void print() {
		
		//PrinterJob object
		PrinterJob printer = PrinterJob.getPrinterJob();
		printer.setPrintable(this);
      
		if(printer.printDialog()) { 
			try {
				//print the document
				printer.print();
			} catch(PrinterException pe) {
				System.out.println("Error: " + pe);
			}
		}
	}

	/**
	 * Print interface
	 */
	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
		
		if(pageIndex > 0) {
			return(NO_SUCH_PAGE);
		} else {
			
	         Graphics2D g2d = (Graphics2D)graphics;
	         g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
	         RepaintManager manager1 = RepaintManager.currentManager(compPrint);
	         manager1.setDoubleBufferingEnabled(false);
	         compPrint.paint(g2d);
	         RepaintManager manager2 = RepaintManager.currentManager(compPrint);
	         manager2.setDoubleBufferingEnabled(true);
	         return(PAGE_EXISTS);
		}
	}
}