import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JMenuItem;
import javax.swing.text.BadLocationException;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class generates all rhymes. A specific word is passed and this
 * class generates monosyllabic, disyllabic and polysyllabic rhyme words.
 * Each search for the rhymes run as a separate thread
 * 
 * @author Daniel Brandstetter
 *
 */
public class Rhyme extends Thread implements ActionListener {

	private Map<String,String> wordlist = new HashMap<String,String>();
	private String word = "";
	private GedichteEditor ge;
	private RhymeTemplateDialog rt;
	private JMenuItem item;
	public List<String> rhyme_one = new ArrayList<String>();
	public List<String> rhyme_two = new ArrayList<String>();
	public List<String> rhyme_more = new ArrayList<String>();	
	
	/**
	 * Constructor
	 * @param ge: access to the main frame  
	 * @param wordlist: list with all dictionary entries
	 * @param word: find rhymes for this word
	 */
	public Rhyme(GedichteEditor ge, Map<String,String> wordlist, String word) {
		this.ge = ge;
		this.wordlist = wordlist;
		this.word = word;
	}	
	
	/**
	 * Constructor
	 * @param rt: acces to the rhyme template frame
	 * @param wordlist: list with all dictionary entries
	 * @param word: find rhymes for this word
	 */
	public Rhyme(RhymeTemplateDialog rt, Map<String,String> wordlist, String word) {
		this.rt = rt;
		this.wordlist = wordlist;
		this.word = word;
	}	
	
	/**
	 * Generate the rhymes for the given word and add it
	 * to rhyme_one, rhyme_two or rhyme_more arraylist.
	 */
	public void handleRhyme () {	
			
		rhyme_one.clear();
		rhyme_two.clear();
		rhyme_more.clear();
		
		String wordValue = "";
		if (wordlist.get(word) != null) wordValue = wordlist.get(word).replaceAll("[^a-zA-Z]", "-");
		else wordValue = word;
		Pattern pattern = Pattern.compile("[aeiou]");
		int index = 0;
		int vindex = 0;
		int index_ = 0;
		
		//Word Process
		String word_lower = word.toLowerCase();
		String word_value_lower = wordValue.toLowerCase();
		String wSplit[] = word_value_lower.split("-");
		
		
		//Iterate
		for (Map.Entry<String, String> entry : wordlist.entrySet()) {
		    
			String key = entry.getKey();
		    String value = entry.getValue();
			String key_lower = key.toLowerCase();
			String value_lower = value.toLowerCase();
		    String vSplit[] = value_lower.split("-");

		    
		    //Male rhyme: The rhyme only extends over one syllable
			if (vSplit.length == 1) {
				if (!(wSplit[wSplit.length-1].substring(wSplit[wSplit.length-1].length()-1).matches("[aeiou]"))) {
					
					index = 0;
		    		vindex = 0;
					
					Matcher matcher = pattern.matcher(wSplit[wSplit.length-1]);
					if (matcher.find()) {
					    index = matcher.start();
					}
					
					//Check h
					vSplit[0] = vSplit[0].replaceAll("h", "");
					wSplit[wSplit.length-1] = wSplit[wSplit.length-1].replaceAll("h", "");				
				
					if (wSplit[index_].length() > 1) vindex = vSplit[0].indexOf(wSplit[wSplit.length-1].substring(index, index+1));
					
					if (vindex >= 0) {
						if (wSplit[wSplit.length-1].substring(Math.max(index, 0)).equals(vSplit[0].substring(Math.max(vindex, 0)))) {
							if (!(word_lower.equals(key_lower))) rhyme_one.add(key);
						}
					}
				}
			}
			
			//Female rhyme: The rhyme extends over two syllable
			if (vSplit.length == 2) {
				if (vSplit[1].equals(wSplit[wSplit.length-1])) {
					
					index = 0;
		    		vindex = 0;
		    		
					if (word.length() >= value.length()) index_ = wSplit.length-2;
					
					Matcher matcher = pattern.matcher(wSplit[index_]);
					if (matcher.find()) {
					    index = matcher.start();
					}
					
					//Check h
					vSplit[0] = vSplit[0].replaceAll("h", "");
					wSplit[index_] = wSplit[index_].replaceAll("h", "");

					if (!(vSplit[0].length() < index)) {
						if (wSplit[index_].length() > 1) vindex = vSplit[0].indexOf(wSplit[index_].substring(index, index+1));
					}
						
					if (vindex >= 0) {
						if (wSplit[index_].substring(Math.max(index, 0)).equals(vSplit[0].substring(Math.max(vindex, 0)))) {
							if (!(word_lower.equals(key_lower))) rhyme_two.add(key);						
						}
					}
				}
				if (wSplit.length == 1) {
				
					index = 0;
		    		vindex = 0;
					
					Matcher matcher = pattern.matcher(wSplit[0]);
					if (matcher.find()) {
					    index = matcher.start();
					}
					
					//Check h
					vSplit[vSplit.length-1] = vSplit[vSplit.length-1].replaceAll("h", "");
					wSplit[0] = wSplit[0].replaceAll("h", "");
					
					if (wSplit[index_].length() > 1) vindex = vSplit[vSplit.length-1].indexOf(wSplit[0].substring(index, index+1));
					
					if (vindex >= 0) {
						if (wSplit[0].substring(Math.max(index, 0)).equals(vSplit[vSplit.length-1].substring(Math.max(vindex, 0)))) {
							if (!(word_lower.equals(key_lower))) rhyme_two.add(key);						
						}
					}
				}
			}
			
			//Rich rhyme: The rhyme extends over three or more syllable
			if (vSplit.length >= 3) {
				if (wSplit.length >= 3) {
					
					if (vSplit[vSplit.length-1].equals(wSplit[wSplit.length-1]) 
							&& vSplit[vSplit.length-2].equals(wSplit[wSplit.length-2])) {

						if (wSplit.length >= vSplit.length) {
							
							index = 0;
						    vindex = 0;
							
							index_ = wSplit.length - vSplit.length;
						
							Matcher matcher = pattern.matcher(wSplit[index_]);
							if (matcher.find()) {
							    index = matcher.start();
							}
							
							//Check h
							vSplit[0] = vSplit[0].replaceAll("h", "");
							wSplit[index_] = wSplit[index_].replaceAll("h", "");
							if (wSplit[index_].length() > 1) vindex = vSplit[0].indexOf(wSplit[index_].substring(index, index+1));
		
							if (vindex >= 0) {
								if (wSplit[index_].substring(Math.max(index, 0)).equals(vSplit[0].substring(Math.max(vindex, 0)))) {
									if (!(word_lower.equals(key_lower))) rhyme_more.add(key);						
								}
							}
						} else if (vSplit.length > wSplit.length) {
							
							index = 0;
						    vindex = 0;
						    int in = 0;							
							in = vSplit.length - wSplit.length;
							
							Matcher matcher = pattern.matcher(wSplit[0]);
							if (matcher.find()) {
							    index = matcher.start();
							}
							
							//Check h
							vSplit[in] = vSplit[in].replaceAll("h", "");
							wSplit[0] = wSplit[0].replaceAll("h", "");
							
							if (wSplit[0].length() > 1) vindex = vSplit[index_].indexOf(wSplit[0].substring(index, index+1));
		
							if (vindex >= 0) {
								if (wSplit[0].substring(Math.max(index, 0)).equals(vSplit[in].substring(Math.max(vindex, 0)))) {
									if (!(word_lower.equals(key_lower))) rhyme_more.add(key);						
								}
							}
						}
					}
				} else if (wSplit.length == 2) {
					
					
					if (vSplit[vSplit.length-1].equals(wSplit[wSplit.length-1])) {
						
						index = 0;
					    vindex = 0;
					    
						Matcher matcher = pattern.matcher(wSplit[0]);
						if (matcher.find()) {
						    index = matcher.start();
						}
						
						//Check h
						vSplit[vSplit.length-2] = vSplit[vSplit.length-2].replaceAll("h", "");
						wSplit[0] = wSplit[0].replaceAll("h", "");
						
						if (wSplit[0].length() > 1) vindex = vSplit[vSplit.length-2].indexOf(wSplit[0].substring(index, index+1));
						
						
						if (vindex >= 0) {
							if (wSplit[0].substring(Math.max(index, 0)).equals(vSplit[vSplit.length-2].substring(Math.max(vindex, 0)))) {
								if (!(word_lower.equals(key_lower))) rhyme_more.add(key);						
							}
						}
					}
					
				} else if (wSplit.length == 1) {
					
					index = 0;
				    vindex = 0;
				    
					Matcher matcher = pattern.matcher(wSplit[0]);
					if (matcher.find()) {
					    index = matcher.start();
					}
					
					
					//Check h
					vSplit[vSplit.length-1] = vSplit[vSplit.length-1].replaceAll("h", "");
					wSplit[0] = wSplit[0].replaceAll("h", "");
					
					if (wSplit[0].length() > 1) vindex = vSplit[vSplit.length-1].indexOf(wSplit[0].substring(index, index+1));
					
					
					if (vindex >= 0) {
						if (wSplit[0].substring(Math.max(index, 0)).equals(vSplit[vSplit.length-1].substring(Math.max(vindex, 0)))) {
							if (!(word_lower.equals(key_lower))) rhyme_more.add(key);						
						}
					}
				}
				
			}
		}
		
		if (ge != null) {

			for (int i = 0; i < this.rhyme_one.size(); i++) {
				
				item = new JMenuItem(this.rhyme_one.get(i));
				item.addActionListener(this);
				ge.subMenuOne.add(item);
				
				item = new JMenuItem(this.rhyme_one.get(i));
				item.addActionListener(this);
				ge.subMenuOne_.add(item);
				
				if (i > 100) break;
			}
			for (int i = 0; i < this.rhyme_two.size(); i++) {
				
				item = new JMenuItem(this.rhyme_two.get(i));
				item.addActionListener(this);
				ge.subMenuTwo.add(item);
				
				item = new JMenuItem(this.rhyme_two.get(i));
				item.addActionListener(this);
				ge.subMenuTwo_.add(item);
				
				if (i > 100) break;
			}
			for (int i = 0; i < this.rhyme_more.size(); i++) {
				
				item = new JMenuItem(this.rhyme_more.get(i));
				item.addActionListener(this);
				ge.subMenuThree.add(item);
				
				item = new JMenuItem(this.rhyme_more.get(i));
				item.addActionListener(this);
				ge.subMenuThree_.add(item);
				
				if (i > 100) break;
			}
			
			if (rhyme_one.size() > 0) ge.subMenuOne.setEnabled(true);
			ge.subMenuOne.setIcon(null);
			if (rhyme_two.size() > 0) ge.subMenuTwo.setEnabled(true);
			ge.subMenuTwo.setIcon(null);
			if (rhyme_more.size() > 0) ge.subMenuThree.setEnabled(true);
			ge.subMenuThree.setIcon(null);
			
			if (rhyme_one.size() > 0) ge.subMenuOne_.setEnabled(true);
			ge.subMenuOne_.setIcon(null);
			if (rhyme_two.size() > 0) ge.subMenuTwo_.setEnabled(true);
			ge.subMenuTwo_.setIcon(null);
			if (rhyme_more.size() > 0) ge.subMenuThree_.setEnabled(true);
			ge.subMenuThree_.setIcon(null);
			
		} else if (rt != null) {

			for (int i = 0; i < this.rhyme_one.size(); i++) {
				
				item = new JMenuItem(this.rhyme_one.get(i));
				item.addActionListener(this);
				rt.subMenuOne.add(item);
				
				if (i > 100) break;
			}
			for (int i = 0; i < this.rhyme_two.size(); i++) {
				
				item = new JMenuItem(this.rhyme_two.get(i));
				item.addActionListener(this);
				rt.subMenuTwo.add(item);
				
				if (i > 100) break;
			}
			for (int i = 0; i < this.rhyme_more.size(); i++) {
				
				item = new JMenuItem(this.rhyme_more.get(i));
				item.addActionListener(this);
				rt.subMenuThree.add(item);
				if (i > 100) break;
			}
			
			if (rhyme_one.size() > 0) rt.subMenuOne.setEnabled(true);
			rt.subMenuOne.setIcon(null);
			if (rhyme_two.size() > 0) rt.subMenuTwo.setEnabled(true);
			rt.subMenuTwo.setIcon(null);
			if (rhyme_more.size() > 0) rt.subMenuThree.setEnabled(true);
			rt.subMenuThree.setIcon(null);

		}
	}
	
	/**
	 * Start the thread to find the rhyme words
	 */
	public void run() {
		
		try {
			handleRhyme();
		} catch (Exception e) {
			System.out.println("Rhyme Error: " + e);
		}
	}

	/**
	 * Handles the click on a rhyme word. 
	 * Also handles in which text field the rhyme word is 
	 * inserted, in the rhyme template dialog.
	 */
	public void actionPerformed(ActionEvent e) {
		
		if (this.ge != null) {
			String text = ge.editor.getText();
			int pos = ge.editor.getCaretPosition();
			int length = ge.editor.getDocument().getLength();
			try {
				text = ge.editor.getDocument().getText(0, length);
			} catch (BadLocationException er) {
				System.out.println(er);
			}
			
			if (text.indexOf("\n", pos) != -1) {
				
				int index = text.indexOf("\n", pos);
				ge.editor.setText(text.substring(0, index)+"\n"+e.getActionCommand()+text.substring(index));
				ge.editor.setCaretPosition(pos);
				
			} else {
				ge.editor.setText(text + "\n" + e.getActionCommand());
				ge.editor.setCaretPosition(pos);
			}

		}
		
		else if (this.rt != null) {
			
			String text = "";
			text = rt.cursorComponent.getText();
			if (text.toString().equals("")) rt.cursorComponent.setText(text + e.getActionCommand());
			else rt.cursorComponent.setText(text + " " + e.getActionCommand());
		}
	}
}
