import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import javax.swing.AbstractAction;
import javax.swing.KeyStroke;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Extends the AbstractAction class to perform undo operations
 * 
 * @author Daniel Brandstetter
 *
 */
class UndoAction extends AbstractAction {
	
	private static final long serialVersionUID = 1L;
	public GedichteEditor ge;
	
	/**
	 * Constructor: Generate undo action
	 * @param ge: main frame
	 */
	public UndoAction(GedichteEditor ge) {
		//super(ge.BUNDLE.getString("GedichteEditor.menu.edit.undo"));
		super("U");
		setEnabled(false);
		
		this.ge = ge;
		this.putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK));
		this.putValue(MNEMONIC_KEY, KeyEvent.VK_R);
	}
	
	/**
	 * Try the undo action
	 */
	public void actionPerformed(ActionEvent e) {
		
		try {
			ge.undoManager.undo();
		} catch (Exception ex) {
	         System.out.println("Error: " + ex);
		}
		
		update();
		ge.redoAction.update();
    }
	
	/**
	 * Update the undo manager, if undo is possible, enable the action
	 */
	protected void update() {
		
		if(ge.undoManager.canUndo()) setEnabled(true);
		else setEnabled(false);
	}
}


/**
 * Extends the AbstractAction class to perform redo operations
 * 
 * @author Daniel Brandstetter
 *
 */
class RedoAction extends AbstractAction {
	
	private static final long serialVersionUID = 1L;
	private GedichteEditor ge;

	/**
	 * Constructor: Generate redo action
	 * @param ge
	 */
	public RedoAction(GedichteEditor ge) {
		//super(ge.BUNDLE.getString("GedichteEditor.menu.edit.redo"));
		super("R");
		setEnabled(false);
		
		this.ge = ge;
		this.putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK));
		this.putValue(MNEMONIC_KEY, KeyEvent.VK_W);
	}
	
	/**
	 * Try the redo action
	 */
	public void actionPerformed(ActionEvent e) {
		
		try {
			ge.undoManager.redo();
		} catch (Exception ex) {
			System.out.println("Error: " + ex);
		}
		
		update();
		ge.undoAction.update();
	}
	
	/**
	 * Update the redo manager, if redo is possible, enable the action
	 */
	protected void update() {
		if(ge.undoManager.canRedo()) setEnabled(true);
		else setEnabled(false);
	}
}