import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.text.BadLocationException;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.JLabel;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class generates the rhyme template dialog. The user can choose 
 * between different templates.
 * 
 * @author Daniel Brandstetter
 *
 */
public class RhymeTemplateDialog extends JDialog implements ActionListener, ItemListener, MouseListener {

	private static final long serialVersionUID = 1L;
	
	private GedichteEditor ge;
	private Dictionary dict;
	
	private final JPanel contentPanel = new JPanel();
	
	@SuppressWarnings("rawtypes")
	public JComboBox comboBox;
	public JTextField tf_rhyme = null;
	public JTextField cursorComponent = null;
	
	public JTextField tf_rhyme_1;
	public JTextField tf_rhyme_2;
	public JTextField tf_rhyme_3;
	public JTextField tf_rhyme_4;
	public JTextField tf_rhyme_5;
	public JTextField tf_rhyme_6;
	public JTextField tf_rhyme_7;
	public JTextField tf_rhyme_8;
	
	private JLabel label_rhyme_1;
	private JLabel label_rhyme_2;
	private JLabel label_rhyme_3;
	private JLabel label_rhyme_4;
	private JLabel label_rhyme_5;
	private JLabel label_rhyme_6;
	private JLabel label_rhyme_7;
	private JLabel label_rhyme_8;

	private JButton btnCancel;
	private JButton btnOk;
	
	private JPopupMenu popupMenu;
	public JMenu subMenuOne;
	public JMenu subMenuTwo;
	public JMenu subMenuThree;
	
	//Variablen
	String oldWordMenu = "";

	/**
	 * Constructor: Create the dialog.
	 * @param ge: access to the main frame
	 * @param dict: dictionary object
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public RhymeTemplateDialog(GedichteEditor ge, Dictionary dict) {
		setResizable(false);
		
		this.ge = ge;
		this.dict = dict;
		
		setTitle(ge.BUNDLE.getString("RhymeTemplateDialog.title"));
		setBounds(100, 100, 450, 362);
		getContentPane().setLayout(null);
		contentPanel.setBounds(0, 0, 434, 291);
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel);
		contentPanel.setLayout(null);
		
		comboBox = new JComboBox();
		comboBox.setModel(new DefaultComboBoxModel(new String[] {
				ge.BUNDLE.getString("RhymeTemplateDialog.fewrhyme"), 
				ge.BUNDLE.getString("RhymeTemplateDialog.crossrhyme"), 
				ge.BUNDLE.getString("RhymeTemplateDialog.huggingrhyme"), 
				ge.BUNDLE.getString("RhymeTemplateDialog.entangledrhyme"), 
				ge.BUNDLE.getString("RhymeTemplateDialog.pilerhyme"), 
				ge.BUNDLE.getString("RhymeTemplateDialog.tailrhyme")
		}));
		
		comboBox.setBounds(10, 11, 414, 20);
		comboBox.addItemListener(this);
		contentPanel.add(comboBox);
		
		tf_rhyme_1 = new JTextField();
		tf_rhyme_1.setName("tf_rhyme_1");
		tf_rhyme_1.setBounds(20, 45, 404, 20);
		contentPanel.add(tf_rhyme_1);
		tf_rhyme_1.setColumns(10);
		tf_rhyme_1.addMouseListener(this);
		
		tf_rhyme_2 = new JTextField();
		tf_rhyme_2.setName("tf_rhyme_2");
		tf_rhyme_2.setColumns(10);
		tf_rhyme_2.setBounds(20, 76, 404, 20);
		contentPanel.add(tf_rhyme_2);
		tf_rhyme_2.addMouseListener(this);
		
		tf_rhyme_3 = new JTextField();
		tf_rhyme_3.setName("tf_rhyme_3");
		tf_rhyme_3.setColumns(10);
		tf_rhyme_3.setBounds(20, 105, 404, 20);
		contentPanel.add(tf_rhyme_3);
		tf_rhyme_3.addMouseListener(this);
		
		tf_rhyme_4 = new JTextField();
		tf_rhyme_4.setName("tf_rhyme_4");
		tf_rhyme_4.setColumns(10);
		tf_rhyme_4.setBounds(20, 134, 404, 20);
		contentPanel.add(tf_rhyme_4);
		tf_rhyme_4.addMouseListener(this);
		
		tf_rhyme_5 = new JTextField();
		tf_rhyme_5.setName("tf_rhyme_5");
		tf_rhyme_5.setColumns(10);
		tf_rhyme_5.setBounds(20, 164, 404, 20);
		contentPanel.add(tf_rhyme_5);
		tf_rhyme_5.addMouseListener(this);
		
		tf_rhyme_6 = new JTextField();
		tf_rhyme_6.setName("tf_rhyme_6");
		tf_rhyme_6.setColumns(10);
		tf_rhyme_6.setBounds(20, 195, 404, 20);
		contentPanel.add(tf_rhyme_6);
		tf_rhyme_6.addMouseListener(this);
		
		tf_rhyme_7 = new JTextField();
		tf_rhyme_7.setName("tf_rhyme_7");
		tf_rhyme_7.setColumns(10);
		tf_rhyme_7.setBounds(20, 226, 404, 20);
		contentPanel.add(tf_rhyme_7);
		tf_rhyme_7.addMouseListener(this);
		
		tf_rhyme_8 = new JTextField();
		tf_rhyme_8.setName("tf_rhyme_8");
		tf_rhyme_8.setColumns(10);
		tf_rhyme_8.setBounds(20, 257, 404, 20);
		contentPanel.add(tf_rhyme_8);
		tf_rhyme_8.addMouseListener(this);
		
		label_rhyme_1 = new JLabel("A");
		label_rhyme_1.setBounds(10, 48, 15, 14);
		contentPanel.add(label_rhyme_1);
		
		label_rhyme_2 = new JLabel("A");
		label_rhyme_2.setBounds(10, 79, 15, 14);
		contentPanel.add(label_rhyme_2);
		
		label_rhyme_3 = new JLabel("B");
		label_rhyme_3.setBounds(10, 108, 15, 14);
		contentPanel.add(label_rhyme_3);
		
		label_rhyme_4 = new JLabel("B");
		label_rhyme_4.setBounds(10, 137, 15, 14);
		contentPanel.add(label_rhyme_4);
		
		label_rhyme_5 = new JLabel("C");
		label_rhyme_5.setBounds(10, 167, 15, 14);
		contentPanel.add(label_rhyme_5);
		
		label_rhyme_6 = new JLabel("C");
		label_rhyme_6.setBounds(10, 198, 15, 14);
		contentPanel.add(label_rhyme_6);
		
		label_rhyme_7 = new JLabel("D");
		label_rhyme_7.setBounds(10, 229, 15, 14);
		contentPanel.add(label_rhyme_7);
		
		label_rhyme_8 = new JLabel("D");
		label_rhyme_8.setBounds(10, 260, 15, 14);
		contentPanel.add(label_rhyme_8);
		
		JPanel buttonPane = new JPanel();
		buttonPane.setBounds(0, 290, 434, 33);
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		getContentPane().add(buttonPane);

		btnOk = new JButton("OK");
		btnOk.addActionListener(this);
		buttonPane.add(btnOk);
		getRootPane().setDefaultButton(btnOk);

		btnCancel = new JButton(ge.BUNDLE.getString("FindDialog.button.cancel.text"));
		btnCancel.addActionListener(this);
		buttonPane.add(btnCancel);
		
		
		//Popup Menu
		popupMenu = new JPopupMenu();

		subMenuOne = new JMenu(ge.BUNDLE.getString("GedichteEditor.menu.rhyme.syllableone"));
		MenuScroller.setScrollerFor(subMenuOne, 10, 100, 0, 0);
		subMenuTwo = new JMenu(ge.BUNDLE.getString("GedichteEditor.menu.rhyme.syllabletwo"));
		MenuScroller.setScrollerFor(subMenuTwo, 10, 100, 0, 0);
		subMenuThree = new JMenu(ge.BUNDLE.getString("GedichteEditor.menu.rhyme.syllablemore"));
		MenuScroller.setScrollerFor(subMenuThree, 10, 100, 0, 0);
		
		subMenuOne.setEnabled(false);
		subMenuTwo.setEnabled(false);
		subMenuThree.setEnabled(false);
		
		popupMenu.add(subMenuOne);
		popupMenu.add(subMenuTwo);
		popupMenu.add(subMenuThree);
		
	}

	/**
	 * Action listener for cancel and OK button
	 */
	public void actionPerformed(ActionEvent e) {

		if(e.getSource() == this.btnCancel) {
			this.dispose();
			this.comboBox.setSelectedIndex(0);
			this.resetAllTfFields();
		}
		
		if(e.getSource() == this.btnOk) {
			this.dispose();
			this.comboBox.setSelectedIndex(0);
			String text = "";
			
			if (ge.editor.getText().isEmpty()) text = "";
			else text = "\n";
			
			if (!tf_rhyme_1.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_1.getText()+"\n";
			if (!tf_rhyme_2.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_2.getText()+"\n";
			if (!tf_rhyme_3.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_3.getText()+"\n";
			if (!tf_rhyme_4.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_4.getText()+"\n";
			if (!tf_rhyme_5.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_5.getText()+"\n";
			if (!tf_rhyme_6.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_6.getText()+"\n";
			if (!tf_rhyme_7.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_7.getText()+"\n";
			if (!tf_rhyme_8.getText().isEmpty() && tf_rhyme_1.isEnabled()) text += this.tf_rhyme_8.getText()+"\n";
			
			this.resetAllTfFields();
			
			text = ge.editor.getText() + text;
			ge.editor.setText(text);
		}
		
	}

	/**
	 * Listener for the template combobox
	 */
	public void itemStateChanged(ItemEvent e) {

		if ((e.getSource() == this.comboBox) && (e.getStateChange() == ItemEvent.SELECTED)) {
			int selectedItem = this.comboBox.getSelectedIndex();
			
			switch (selectedItem) {
				case 0:
					this.label_rhyme_1.setText("A");
					this.label_rhyme_2.setText("A");
					this.label_rhyme_3.setText("B");
					this.label_rhyme_4.setText("B");
					this.label_rhyme_5.setText("C");
					this.label_rhyme_6.setText("C");
					this.label_rhyme_7.setText("D");
					this.label_rhyme_8.setText("D");
					this.enableAllTfFields();
					break;
				case 1:
					this.label_rhyme_1.setText("A");
					this.label_rhyme_2.setText("B");
					this.label_rhyme_3.setText("A");
					this.label_rhyme_4.setText("B");
					this.label_rhyme_5.setText("C");
					this.label_rhyme_6.setText("D");
					this.label_rhyme_7.setText("C");
					this.label_rhyme_8.setText("D");
					this.enableAllTfFields();
					break;
				case 2:
					this.label_rhyme_1.setText("A");
					this.label_rhyme_2.setText("B");
					this.label_rhyme_3.setText("B");
					this.label_rhyme_4.setText("A");
					this.label_rhyme_5.setText("C");
					this.label_rhyme_6.setText("D");
					this.label_rhyme_7.setText("D");
					this.label_rhyme_8.setText("C");
					this.enableAllTfFields();
					break;
				case 3:
					this.enableAllTfFields();
					this.label_rhyme_1.setText("A");
					this.label_rhyme_2.setText("B");
					this.label_rhyme_3.setText("C");
					this.label_rhyme_4.setText("A");
					this.label_rhyme_5.setText("B");
					this.label_rhyme_6.setText("C");
					this.label_rhyme_7.setText("");
					this.label_rhyme_8.setText("");
					this.tf_rhyme_7.setEnabled(false);
					this.tf_rhyme_8.setEnabled(false);
					break;
				case 4:
					this.enableAllTfFields();
					this.label_rhyme_1.setText("A");
					this.label_rhyme_2.setText("A");
					this.label_rhyme_3.setText("A");
					this.label_rhyme_4.setText("A");
					this.label_rhyme_5.setText("B");
					this.label_rhyme_6.setText("B");
					this.label_rhyme_7.setText("B");
					this.label_rhyme_8.setText("B");
					this.enableAllTfFields();
					break;
				case 5:
					this.enableAllTfFields();
					this.label_rhyme_1.setText("A");
					this.label_rhyme_2.setText("A");
					this.label_rhyme_3.setText("B");
					this.label_rhyme_4.setText("C");
					this.label_rhyme_5.setText("C");
					this.label_rhyme_6.setText("B");
					this.label_rhyme_7.setText("");
					this.label_rhyme_8.setText("");
					this.tf_rhyme_7.setEnabled(false);
					this.tf_rhyme_8.setEnabled(false);
					break;
			}
		}	
	}
	
	/**
	 * Enable all textfields
	 */
	public void enableAllTfFields() {
		
		this.tf_rhyme_1.setEnabled(true);
		this.tf_rhyme_2.setEnabled(true);
		this.tf_rhyme_3.setEnabled(true);
		this.tf_rhyme_4.setEnabled(true);
		this.tf_rhyme_5.setEnabled(true);
		this.tf_rhyme_6.setEnabled(true);
		this.tf_rhyme_7.setEnabled(true);
		this.tf_rhyme_8.setEnabled(true);
	}
	
	/**
	 * Reset all textfields ->set no text
	 */
	public void resetAllTfFields() {
		
		this.tf_rhyme_1.setText("");
		this.tf_rhyme_2.setText("");
		this.tf_rhyme_3.setText("");
		this.tf_rhyme_4.setText("");
		this.tf_rhyme_5.setText("");
		this.tf_rhyme_6.setText("");
		this.tf_rhyme_7.setText("");
		this.tf_rhyme_8.setText("");
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON1) {
			if (e.getSource() == tf_rhyme_1) {
				cursorComponent = tf_rhyme_1;
			} else if (e.getSource() == tf_rhyme_2) {
				cursorComponent = tf_rhyme_2;
			} else if (e.getSource() == tf_rhyme_3) {
				cursorComponent = tf_rhyme_3;
			} else if (e.getSource() == tf_rhyme_4) {
				cursorComponent = tf_rhyme_4;
			} else if (e.getSource() == tf_rhyme_5) {
				cursorComponent = tf_rhyme_5;
			} else if (e.getSource() == tf_rhyme_6) {
				cursorComponent = tf_rhyme_6;
			} else if (e.getSource() == tf_rhyme_7) {
				cursorComponent = tf_rhyme_7;
			} else if (e.getSource() == tf_rhyme_8) {
				cursorComponent = tf_rhyme_8;
			}
		}
	}
	@Override
	public void mouseEntered(MouseEvent arg0) {		
	}
	@Override
	public void mouseExited(MouseEvent arg0) {		
	}
	@Override
	public void mousePressed(MouseEvent arg0) {
	}

	/**
	 * Generate the rhyme words on mouse released
	 */
	public void mouseReleased(MouseEvent e) {

		if (e.getButton() == MouseEvent.BUTTON3) {
			
			
			//Variablen
			String word = "";
			
			if (e.getSource() == tf_rhyme_1) {
				tf_rhyme = tf_rhyme_1;
			} else if (e.getSource() == tf_rhyme_2) {
				tf_rhyme = tf_rhyme_2;
			} else if (e.getSource() == tf_rhyme_3) {
				tf_rhyme = tf_rhyme_3;
			} else if (e.getSource() == tf_rhyme_4) {
				tf_rhyme = tf_rhyme_4;
			} else if (e.getSource() == tf_rhyme_5) {
				tf_rhyme = tf_rhyme_5;
			} else if (e.getSource() == tf_rhyme_6) {
				tf_rhyme = tf_rhyme_6;
			} else if (e.getSource() == tf_rhyme_7) {
				tf_rhyme = tf_rhyme_7;
			} else if (e.getSource() == tf_rhyme_8) {
				tf_rhyme = tf_rhyme_8;
			}
			
			//tf_rhyme.setCaretPosition(tf_rhyme.viewToModel(e.getPoint()));
			word = getSelectedWord(tf_rhyme);			
			
			if (word != null && !word.equals("") && !word.equals(" ") && !oldWordMenu.equals(word)) {
				
				subMenuOne.removeAll();
				subMenuTwo.removeAll();
				subMenuThree.removeAll();
				
				subMenuOne.setEnabled(false);
				subMenuOne.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				subMenuTwo.setEnabled(false);
				subMenuTwo.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				subMenuThree.setEnabled(false);
				subMenuThree.setIcon(new ImageIcon(this.getClass().getResource("/resource/loading.gif")));
				
				Rhyme r = new Rhyme(this, dict.getWordlist(), word);
				r.start();
				
				oldWordMenu = word;
					
			}
			
			//Popup Menu
			popupMenu.show(tf_rhyme, e.getX()+10, e.getY()+10);
			subMenuOne.setMenuLocation(subMenuOne.getWidth()+2, 0);
			subMenuTwo.setMenuLocation(subMenuTwo.getWidth()+2, 0);
			subMenuThree.setMenuLocation(subMenuThree.getWidth()+2, 0);
		}		
	}
	
	/**
	 * Get the word, for which the rhymes are to be formed
	 * @param tf_rhyme: textfield, in which the word is
	 * @return the looking word
	 */
	private String getSelectedWord(JTextField tf_rhyme) {
		
		int pos = tf_rhyme.getCaretPosition();
		int start = pos;
		int end = pos;
		String word = "";
		
		try {
			if (start > 0) start -= 1;
			while ((tf_rhyme.getText(start, 1).matches("[a-zA-Z]+")) && (start > 0)) start -= 1;
			while ((tf_rhyme.getText(end, 1).matches("[a-zA-Z]+")) && (end < (tf_rhyme.getText().length()))) end += 1;
			
			word = tf_rhyme.getText(start, end-start).trim();
			word = word.replaceAll("[^a-zA-Z]", "");
			
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}

		return word;
	}
}
